﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Xml.Serialization;
using DSInterfaces;
using System.Security;
using System.Security.Principal;
using Microsoft.Win32;
using System.Collections;


namespace erodnet
{
    public partial class MainForm : Form
    {
        [DllImport("shell32.dll", CharSet = CharSet.Auto, EntryPoint = "SHAddToRecentDocs")]
        internal static extern void SHAddToRecentDocs(UInt32 uFlags, string pv);

        [DllImport("oleaut32.dll", CharSet = CharSet.Unicode, ExactSpelling = true)]
        public static extern int OleCreatePropertyFrame(
            [In] IntPtr hwndOwner,
            [In] int x,
            [In] int y,
            [In, MarshalAs(UnmanagedType.LPWStr)] string lpszCaption,
            [In] int cObjects,
            [In, MarshalAs(UnmanagedType.LPArray, ArraySubType = UnmanagedType.IUnknown)] object[] ppUnk,
            [In] int cPages,
            [In] IntPtr pPageClsID,
            [In] int lcid,
            [In] int dwReserved,
            [In] IntPtr pvReserved
            );

        // SHGetFileInfo関数
        [DllImport("shell32.dll")]
        private static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, uint cbSizeFileInfo, uint uFlags);

        // SHGetFileInfo関数で使用するフラグ
        private const uint SHGFI_ICON = 0x100; // アイコン・リソースの取得
        private const uint SHGFI_LARGEICON = 0x0; // 大きいアイコン
        private const uint SHGFI_SMALLICON = 0x1; // 小さいアイコン

        // SHGetFileInfo関数で使用する構造体
        private struct SHFILEINFO
        {
            public IntPtr hIcon;
            public IntPtr iIcon;
            public uint dwAttributes;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
            public string szDisplayName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
            public string szTypeName;
        };

        [DllImport("user32.dll")]
        extern public static int RegisterWindowMessage(string sRwm);
        public static int WM_DSEvent = RegisterWindowMessage(Application.ExecutablePath);

        const int WM_SYSCOMMAND = 0x0112;
        const int SC_MAXIMIZE = 61488;

        FormLib fl = null;

        ArrayList fileHistList1;
        ArrayList fileHistList2;
        ArrayList acceptedList;
        ArrayList playList;
        ArrayList timeList;

        bool trackBarTracking = false;

        int MouseX = 0;
        int MouseY = 0;

        bool hideMouse = false;

        int apTop, apLeft, apHeight, apWidth;

        string currentFile = "";

        IBaseFilter[] currentFilters;

        int videoWidth = 0;
        int videoHeight = 0;

        string[] MenuKey ={"0","1","2","3","4","5","6","7","8","9",
                               "A","B","C","D","E","F","G","H","I","J",
                               "K","L","M","N","O","P","Q","R","S","T",
                               "U","V","W","X","Y"};
        bool lastRender = false;

        public MainForm()
        {
            InitializeComponent();
            this.Text = Title.LoadInfo();
            // アプリケーション・アイコンを取得
            SHFILEINFO shinfo = new SHFILEINFO();
            IntPtr hSuccess = SHGetFileInfo(Application.ExecutablePath, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo), SHGFI_ICON | SHGFI_LARGEICON);
            if (hSuccess != IntPtr.Zero)
            {
                this.Icon = Icon.FromHandle(shinfo.hIcon);
            }

            //Icon = new Icon(Application.ExecutablePath, "Default.ico");
            // We paint the windows ourself
            this.SetStyle(ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
            fl = new FormLib(this);
            fl.Complete += this.OnComplete;
            fileHistList1 = new ArrayList();
            fileHistList2 = new ArrayList();
            acceptedList = new ArrayList();
            playList = new ArrayList();
            timeList = new ArrayList();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ReadXML();
            ShowDigiCount();
            timerGeneral.Enabled = true;

            string[] ss = System.Environment.GetCommandLineArgs();
            for (int i = 1; i < ss.Length; i++)
            {
                AcceptFile(ss[i]);
            }
        }

        protected override void WndProc(ref Message m)
        {
            if ((m.Msg == WM_SYSCOMMAND) && (m.WParam == (IntPtr)SC_MAXIMIZE))
            {
                DoFullScreen();
                m.Result = IntPtr.Zero;
                //return;
            }
            if (m.Msg == WM_DSEvent)
            {
                fl.HandleEvent();
                m.Result = IntPtr.Zero;
            }
            if (m.Msg == Program.WM_SETTEXT)
            {
                AcceptFile(Marshal.PtrToStringAuto(m.LParam));
                m.Result = IntPtr.Zero;
                return;
            }
            base.WndProc(ref m);
        }

        private void OnComplete(object sender, EventArgs e)
        {
            int num = playList.IndexOf(currentFile);
            if (num >= 0)//プレイリスト再生の時は別処理
            {
                //一曲終わった時、次の曲を再生させる
                if (num < playList.Count - 1) AcceptFile((string)playList[num + 1]);
                //ループ指定してたら、最後の曲が終わって最初の曲に戻って再生
                else if (buttonLoop.Checked) AcceptFile((string)playList[0]);
                //ループ指定してない時は、最初の曲の先頭に戻るが再生はしない
                else
                {
                    AcceptFile((string)playList[0]);
                    DoAccept();
                }
                return;
            }
            if (buttonLoop.Checked)
            {
                fl.Position = 0;
                InitTrackBar();
            }
            else
            {
                fl.PauseGraph();
                menuStop_Click(null, null);
            }
        }

        private void menuOpen_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                AcceptFile(openFileDialog1.FileName);
            }
        }


        private void menuClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void DoFullScreen()
        {
            if (WindowState == FormWindowState.Maximized)
            {
                WindowState = FormWindowState.Normal;
                FormBorderStyle = FormBorderStyle.Sizable;
                panelAll.Visible = true;
            }
            else
            {
                FormBorderStyle = FormBorderStyle.None;
                WindowState = FormWindowState.Maximized;
                BringToFront();//これはほとんどうまく行かない
                contextMenuStripMax.Show();//左クリックで前面に出ない場合でも、右クリックで前面に出る。
                contextMenuStripMax.Hide();//メニューからフォーカスを取り戻す。
            }
            buttonFullScreen.Checked = WindowState == FormWindowState.Maximized;
        }

        private void Form1_DoubleClick(object sender, EventArgs e)
        {
            DoFullScreen();
        }

        private void listBox1_DragDrop(object sender, DragEventArgs e)
        {

        }
        /*
本体へは複数ドロップしても単品扱い

ビデオはリストをクリアして再生
音楽は
リストにあるファイルは再生
リストにないファイルはリストクリアして再生（窓にドロップはリスト追加）


リストへのドロップは音楽だけリスト追加、再生に関与しない
         */

        private bool AcceptPlayList(string fileName)
        {
            if (fileName != "")
            {
                string s = System.IO.Path.GetFullPath(fileName);
                if (File.Exists(s))
                {
                    bool isAudio, isVideo;
                    long audioLength;
                    fl.GetMediaContents(s, out isVideo, out isAudio, out audioLength);
                    if (isAudio && (!isVideo))
                    {
                        if (fileHistList2.Contains(s))
                        {
                            fileHistList2.RemoveAt(fileHistList2.IndexOf(s));
                        }
                        fileHistList2.Insert(0, s);
                        if (playList.Contains(s))
                        {
                        }
                        else
                        {
                            playList.Add(s);
                            timeList.Add(audioLength);
                            listBox1.Items.Add(Path.GetFileName(s));
                            if (listBox1.Items.Count > 1)
                            {
                            }
                            else
                            {
                                listBox1.SelectedIndex = 0;
                            }
                        }
                        return true;
                    }
                }
            }
            return false;
        }

        private void AcceptFile(string fileName)
        {
            if ((fileName == "") || (fileName == Title.LoadInfo())) return;
            string s = System.IO.Path.GetFullPath(fileName);
            if (!File.Exists(s)) return;
            if (acceptedList.IndexOf(s) < 0)
            {
                acceptedList.Add(s);
            }
        }

        private void DoAccept()
        {
            string s = (string)acceptedList[0];
            acceptedList.RemoveAt(0);
            bool isAudio, isVideo;
            long audioLength;
            fl.GetMediaContents(s, out isVideo, out isAudio,out audioLength);
            if (isAudio && (!isVideo))
            {
                if (fileHistList2.Contains(s))
                {
                    fileHistList2.RemoveAt(fileHistList2.IndexOf(s));
                }
                fileHistList2.Insert(0, s);
                if (playList.Contains(s))
                {
                    listBox1.SelectedIndex = playList.IndexOf(s);
                }
                else
                {
                    listBox1.Items.Clear();
                    playList.Clear();
                    timeList.Clear();

                    playList.Add(s);
                    timeList.Add(audioLength);
                    listBox1.Items.Add(Path.GetFileName(s));
                    listBox1.SelectedIndex = 0;
                }
            }
            else
            {
                if (fileHistList1.Contains(s))
                {
                    fileHistList1.RemoveAt(fileHistList1.IndexOf(s));
                }
                fileHistList1.Insert(0, s);
                listBox1.Items.Clear();
                playList.Clear();
                timeList.Clear();
            }
            panelList.Visible = listBox1.Items.Count > 0;
            fl.CloseInterfaces();
            currentFile = s;
            listBox1_SelectedIndexChanged(null, null);
            Invalidate();
            if (fl.BuildGraph(currentFile, isVideo))
            {
                InitTrackBar();
                menuInfo.Enabled = true;
                fl.Volume = trackBarVolume.Value;
                listBox1.Invalidate();
                ShowDigiCount();
                InitMediaInfo();
                SHAddToRecentDocs(3, currentFile);
                /*
  
Application.RestoreTopMosts;
Application.Restore;
  
                 */
                lastRender = true;
            }
            else
            {
                InitTrackBar();
                menuInfo.Enabled = true;
                ShowDigiCount();
                InitMediaInfo();
                listBox1.Invalidate();
                lastRender = false;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (acceptedList.Count > 0)
            {
                lastRender = false;
                while (acceptedList.Count > 0)
                {
                    DoAccept();
                }
                if (lastRender)
                {
                    lastRender = false;
                    BringToFront();
                    fl.RunGraph();
                    WriteXML();
                }
                return;//ファイルを受け付けてすぐのタイマーイベントは回避しよう
            }
            if (fl.Active)
            {
                if (fl.State == FilterState.Running)
                {
                    if (!trackBarTracking)
                    {
                        trackBar1.Value = Math.Max(0, Math.Min(trackBar1.Maximum,(int)((fl.Position * (ulong)trackBar1.Maximum) / Math.Max(fl.StopPosition, (ulong)1))));
                    }
                    ShowDigiCount();
                }
            }
            else
            {
            }
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            if (hideMouse)
            {
                System.Windows.Forms.Cursor.Show();
                hideMouse = false;
            }
            menuLoop.Checked = buttonLoop.Checked;
            menuParent.Enabled = currentFile != "";
            menuTopMost.Checked = this.TopMost;
            menuFullScreen.Checked = WindowState == FormWindowState.Maximized;
            menuInfo.DropDownItems.Clear();
            if (fl.EnumFilters(out currentFilters))
            {
                menuInfo.Enabled = true;
                for (int i = 0; i < currentFilters.Length; i++)
                {
                    FilterInfo info;
                    currentFilters[i].QueryFilterInfo(out info);
                    ToolStripMenuItem menuItem = new ToolStripMenuItem(info.achName);
                    menuItem.Tag = i;
                    menuItem.Click += new EventHandler(CodecClick);
                    try
                    {
                        ISpecifyPropertyPages pages = (ISpecifyPropertyPages)currentFilters[i];
                    }
                    catch
                    {
                        menuItem.Enabled = false;
                    }
                    menuInfo.DropDownItems.Add(menuItem);
                }
            }
            else menuInfo.Enabled = false;

            int ii = 0;
            while (ii < fileHistList1.Count)
            {
                if (File.Exists((string)fileHistList1[ii]))
                {
                    ii++;
                }
                else
                {
                    fileHistList1.RemoveAt(ii);
                }
            }
            ii = 0;
            while (ii < fileHistList2.Count)
            {
                if (File.Exists((string)fileHistList2[ii]))
                {
                    ii++;
                }
                else
                {
                    fileHistList2.RemoveAt(ii);
                }
            }

            menuHist1.Visible = fileHistList1.Count > 0;
            menuHist1.DropDownItems.Clear();
            for (int i = 0; i < 35; i++)
            {
                if (fileHistList1.Count > i)
                {
                    ToolStripMenuItem menuItem = new ToolStripMenuItem();
                    menuItem.ToolTipText = (string)fileHistList1[i];
                    menuItem.Text = "&" + MenuKey[i] + ")" + Path.GetFileName(menuItem.ToolTipText);
                    menuItem.Click += new EventHandler(menu0_Y_Click);
                    menuHist1.DropDownItems.Add(menuItem);
                }
            }
            menuHist1.DropDownItems.Add(menuSplit1);
            menuHist1.DropDownItems.Add(menuDelHist1);

            menuHist2.Visible = fileHistList2.Count > 0;
            menuHist2.DropDownItems.Clear();
            for (int i = 0; i < 35; i++)
            {
                if (fileHistList2.Count > i)
                {
                    ToolStripMenuItem menuItem = new ToolStripMenuItem();
                    menuItem.ToolTipText = (string)fileHistList2[i];
                    menuItem.Text = "&" + MenuKey[i] + ")" + Path.GetFileName(menuItem.ToolTipText);
                    menuItem.Click += new EventHandler(menu0_Y_Click);
                    menuHist2.DropDownItems.Add(menuItem);
                }
            }
            menuHist2.DropDownItems.Add(menuSplit2);
            menuHist2.DropDownItems.Add(menuDelHist2);

            menuPlayList.Visible = panelList.Visible;

            menuHist3.Visible = fileHistList2.Count > 0;
            menuHist3.DropDownItems.Clear();
            for (int i = 0; i < 35; i++)
            {
                if (fileHistList2.Count > i)
                {
                    ToolStripMenuItem menuItem = new ToolStripMenuItem();
                    menuItem.ToolTipText = (string)fileHistList2[i];
                    menuItem.Text = "&" + MenuKey[i] + ")" + Path.GetFileName(menuItem.ToolTipText);
                    menuItem.Click += new EventHandler(menuPlayList0_Y_Click);
                    menuHist3.DropDownItems.Add(menuItem);
                }
            }
            //menuHist3.DropDownItems.Add(menuSplit2);//一つの項目を複数の場所に挿入することは出来ないらしい
            //menuHist3.DropDownItems.Add(menuDelHist2);//よって、プレイリストの方には挿入しない
        }

        private void menu0_Y_Click(object sender, EventArgs e)
        {
            AcceptFile(((ToolStripMenuItem)sender).ToolTipText);
        }

        private void WriteXML()
        {
            string pathname = Path.ChangeExtension(Application.ExecutablePath, ".xml");
            AppProp ap = new AppProp();
            for (int i = 0; i < ap.Hist1.Length; i++)
            {
                if (fileHistList1.Count > i)
                {
                    ap.Hist1[i] = (string)fileHistList1[i];
                }
            }
            for (int i = 0; i < ap.Hist2.Length; i++)
            {
                if (fileHistList2.Count > i)
                {
                    ap.Hist2[i] = (string)fileHistList2[i];
                }
            }
            ap.Loop = buttonLoop.Checked;
            ap.Volume = trackBarVolume.Value;
            ap.Top = apTop;
            ap.Left = apLeft;
            ap.Height = apHeight;
            ap.Width = apWidth;
            ap.TopMost = this.TopMost;

            XmlSerializer XML = new XmlSerializer(typeof(AppProp));
            StreamWriter sw = new StreamWriter(pathname);
            try
            {
                XML.Serialize(sw, ap);
            }
            finally
            {
                sw.Close();
            }
        }

        private void ReadXML()
        {
            string pathname = Path.ChangeExtension(Application.ExecutablePath, ".xml");
            if (!File.Exists(pathname)) return;
            AppProp ap = new AppProp();
            ap.Top = apTop;
            ap.Left = apLeft;
            ap.Height = apHeight;
            ap.Width = apWidth;
            XmlSerializer XML = new XmlSerializer(typeof(AppProp));
            StreamReader sr = new StreamReader(pathname);
            try
            {
                ap = (AppProp)XML.Deserialize(sr);
            }
            catch { return; }
            finally
            {
                sr.Close();
            }
            for (int i = 0; i < ap.Hist1.Length; i++)
            {
                if (ap.Hist1[i].Length > 0) fileHistList1.Add(ap.Hist1[i]);
            }
            for (int i = 0; i < ap.Hist2.Length; i++)
            {
                if (ap.Hist2[i].Length > 0) fileHistList2.Add(ap.Hist2[i]);
            }
            buttonLoop.Checked = ap.Loop;
            trackBarVolume.Value = ap.Volume;
            this.Top =Math.Max(0, ap.Top);
            this.Left =Math.Max(0, ap.Left);
            this.Height = ap.Height;
            this.Width = ap.Width;
            this.TopMost = ap.TopMost;
            buttonTopMost.Checked = this.TopMost;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (fl.Active)
            {
                CloseFile();
                e.Cancel = true;
            }
            else
            {
                WriteXML();
            }
        }

        private void CloseFile()
        {
            listBox1.Items.Clear();
            playList.Clear();
            timeList.Clear();
            panelList.Hide();
            fl.StopGraph();
            fl.CloseInterfaces();
            menuInfo.Enabled = false;
            currentFile = "";
            InitMediaInfo();
            videoHeight = 0;
            videoWidth = 0;
            InitTrackBar();
            ShowDigiCount();
            Invalidate();
        }

        private void InitTrackBar()
        {
            trackBar1.Enabled = fl.Active;
            trackBar1.Value = 0;
        }

        private void OnTracked()
        {
            if (fl.Active)
            {
                fl.Position = ((ulong)trackBar1.Value * fl.StopPosition) / (ulong)trackBar1.Maximum;
                ShowDigiCount();
            }
        }

        private void trackBar1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left) trackBarTracking = true;
        }

        private void trackBar1_MouseUp(object sender, MouseEventArgs e)
        {
            if ((e.Button == MouseButtons.Left) && trackBarTracking)
            {
                trackBarTracking = false;
                OnTracked();
            }
        }

        private void ShowDigiCount()
        {
            if (!fl.Active)
            {
                digiLabel.Text = "";
                return;
            }
            if (fl.State == FilterState.Stopped)
            {
                digiLabel.Text = "停止中\n" + Path.GetFileName(currentFile);
                //digiToolTip.SetToolTip(digiPanel, "");
                return;
            }
            fl.GetVideoSize(out videoWidth, out videoHeight);

            int ii = (int)(fl.Position / 10000000);
            int h = ii / 3600;
            int m = (ii % 3600) / 60;
            int s = (ii % 3600) % 60;
            string now = String.Format("{0:D2}:{1:D2}:{2:D2}", h, m, s);
            ii = (int)(fl.StopPosition / 10000000);
            h = ii / 3600;
            m = (ii % 3600) / 60;
            s = (ii % 3600) % 60;
            string total = String.Format("{0:D2}:{1:D2}:{2:D2}", h, m, s);
            string ws = now.ToString() + "/" + total.ToString() + "\n" + Path.GetFileName(currentFile) + "\n";
            if ((videoWidth > 0) && (videoHeight > 0))
            {
                ws += videoWidth.ToString() + "x" + videoHeight.ToString();
            }
            digiLabel.Text = ws;
        }

        private void menuSnapshot_Click(object sender, EventArgs e)
        {
            Stream buffer;
            if (fl.GetBitmap(out buffer))
            {
                SnapForm sf = new SnapForm(buffer);
                sf.Show();
            }
        }

        private void menuStop_Click(object sender, EventArgs e)
        {
            if (!fl.Active) return;
            if (fl.State == FilterState.Running)
            {
                fl.PauseGraph();
            }
            else
            {
                fl.Position = (uint)0;
                fl.StopGraph();
                InitTrackBar();
            }
        }

        private void menuPlay_Click(object sender, EventArgs e)
        {
            if (!fl.Active) return;
            if (fl.State == FilterState.Running)
            {
                fl.PauseGraph();
            }
            else
            {
                fl.RunGraph();
            }
        }

        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Location.X == MouseX) && (e.Location.Y == MouseY))
            {
                timerMouse.Enabled = true;
            }
            else
            {
                MouseX = e.Location.X;
                MouseY = e.Location.Y;
                if (hideMouse)
                {
                    System.Windows.Forms.Cursor.Show();
                    hideMouse = false;
                }
                panelAll.Visible = true;
                timerMouse.Enabled = false;
                timerMouse.Enabled = true;
            }
        }

        private void timerMouse_Tick(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Maximized)
            {
                panelAll.Visible = false;
            }
            if (!hideMouse)
            {
                System.Windows.Forms.Cursor.Hide();
                hideMouse = true;
            }
            timerMouse.Enabled = false;
        }

        private void Form1_MouseLeave(object sender, EventArgs e)
        {
            timerMouse.Enabled = false;
        }

        private void menuParent_Click(object sender, EventArgs e)
        {
            if (currentFile == "") return;
            string s = currentFile;
            string ss = Path.GetFileName(s);
            s = s.Remove(s.Length - ss.Length);
            System.Diagnostics.Process.Start(s);
        }

        private void menuTopMost_Click(object sender, EventArgs e)
        {
            this.TopMost = !this.TopMost;
            buttonTopMost.Checked = this.TopMost;
        }

        private void menuFullScreen_Click(object sender, EventArgs e)
        {
            DoFullScreen();
        }

        private void menuRelate_Click(object sender, EventArgs e)
        {
            bool isAdmin = (new WindowsPrincipal(WindowsIdentity.GetCurrent())).IsInRole(@"BUILTIN\Administrators");
            if ((!isAdmin) && (MessageBox.Show("関連付けを行うには管理者権限が必要です。\n警告が表示されたり、パスワードが要求されたりします。",
                Title.AppTitle, MessageBoxButtons.OKCancel) != DialogResult.OK)) return;
            if (this.TopMost) menuTopMost_Click(null, null);
            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.UseShellExecute = true;
            startInfo.WorkingDirectory = Environment.CurrentDirectory;
            startInfo.FileName = Application.ExecutablePath;
            if (!isAdmin) startInfo.Verb = "runas";
            startInfo.Arguments = "/Setup";

            try
            {
                Process p = Process.Start(startInfo);
            }
            catch
            {
                return;
            }
        }

        private void menuHome_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://wantech.ikuto.com/");
        }

        private void menuDelHist1_Click(object sender, EventArgs e)
        {
            HistForm f = new HistForm();
            for (int i = 0; i < fileHistList1.Count; i++)
            {
                f.checkedListBox1.Items.Add(fileHistList1[i]);
            }
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                fileHistList1.Clear();
                for (int i = 0; i < f.checkedListBox1.Items.Count; i++)
                {
                    if (!f.checkedListBox1.GetItemChecked(i))
                    {
                        fileHistList1.Add(f.checkedListBox1.Items[i]);
                    }
                }
                WriteXML();
            }
        }

        private void menuDelHist2_Click(object sender, EventArgs e)
        {
            HistForm f = new HistForm();
            for (int i = 0; i < fileHistList2.Count; i++)
            {
                f.checkedListBox1.Items.Add(fileHistList2[i]);
            }
            if (f.ShowDialog(this) == DialogResult.OK)
            {
                fileHistList2.Clear();
                for (int i = 0; i < f.checkedListBox1.Items.Count; i++)
                {
                    if (!f.checkedListBox1.GetItemChecked(i))
                    {
                        fileHistList2.Add(f.checkedListBox1.Items[i]);
                    }
                }
                WriteXML();
            }
        }

        private void menuLoop_Click(object sender, EventArgs e)
        {
            buttonLoop.Checked = !buttonLoop.Checked;
        }

        private void menuSpeedDown_Click(object sender, EventArgs e)
        {
            fl.SpeedDown();
        }

        private void menuSpeedUp_Click(object sender, EventArgs e)
        {
            fl.SpeedUp();
        }

        private void trackBarVolume_Scroll(object sender, EventArgs e)
        {
            fl.Volume = trackBarVolume.Value;
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Normal)
            {
                apTop = this.Top;
                apLeft = this.Left;
                apHeight = this.Height;
                apWidth = this.Width;
            }
        }

        private void CodecClick(object sender, EventArgs e)
        {
            int i = (int)(((ToolStripMenuItem)sender).Tag);
            if (currentFilters.Length <= i) return;
            IBaseFilter filter = currentFilters[i];
            DsCAUUID pages = new DsCAUUID();
            try
            {
                ((ISpecifyPropertyPages)filter).GetPages(out pages);
                object[] o = new object[1];
                o[0] = currentFilters[i];
                OleCreatePropertyFrame(this.Handle, 0, 0, ((ToolStripMenuItem)sender).Text, 1, o, pages.cElems, pages.pElems, 0, 0, (IntPtr)0);
            }
            catch
            {
            }
            finally
            {
                if (pages.pElems != null) Marshal.FreeCoTaskMem(pages.pElems);
            }
        }

        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                foreach (string fileName in (string[])e.Data.GetData(DataFormats.FileDrop))
                    AcceptFile(fileName);
            }
        }

        private void MainForm_KeyPress(object sender, KeyPressEventArgs e)
        {
            switch (e.KeyChar)
            {
                case (char)Keys.Escape:
                    if (this.WindowState == FormWindowState.Maximized)
                    {
                        DoFullScreen();
                    }
                    break;
            }
        }

        private void InitMediaInfo()
        {
            string s = fl.GetMediaInfoString(currentFile);
            digiToolTip.SetToolTip(digiPanel, s);
            digiToolTip.SetToolTip(digiLabel, s);
        }

        private void listBox1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int y = (e.Y / listBox1.ItemHeight);
            if (listBox1.Items.Count < y + 1) DoFullScreen();
            else
            {
                AcceptFile((string)playList[y]);
            }
        }

        private void listBox1_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                int y = (e.Y / listBox1.ItemHeight);
                if (listBox1.Items.Count < y + 1)
                {
                    contextMenuStripMain.Show(listBox1, e.X, e.Y);
                }
                else
                {
                    contextMenuStripPlayList.Show(listBox1, e.X, e.Y);
                }
            }
        }

        private void listBox1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                int y = (e.Y / listBox1.ItemHeight);
                if (listBox1.Items.Count < y + 1)
                {

                }
                else
                {
                    listBox1.SelectedIndex = y;
                }
            }

        }

        private void menuAddPlayList_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                if(!AcceptPlayList(openFileDialog1.FileName))AcceptFile(openFileDialog1.FileName);
            }
        }

        private void splitButtonAdd_DropDownOpening(object sender, EventArgs e)
        {
            menuHistPlayList.Visible = fileHistList2.Count > 0;
            menuHistPlayList.DropDownItems.Clear();
            for (int i = 0; i < 35; i++)
            {
                if (fileHistList2.Count > i)
                {
                    ToolStripMenuItem menuItem = new ToolStripMenuItem();
                    menuItem.ToolTipText = (string)fileHistList2[i];
                    menuItem.Text = "&" + MenuKey[i] + ")" + Path.GetFileName(menuItem.ToolTipText);
                    menuItem.Click += new EventHandler(menuPlayList0_Y_Click);
                    menuHistPlayList.DropDownItems.Add(menuItem);
                }
            }
            menuHistPlayList.DropDownItems.Add(menuSplit2);
            menuHistPlayList.DropDownItems.Add(menuDelHist2);
        }

        private void menuPlayList0_Y_Click(object sender, EventArgs e)
        {
            string s = ((ToolStripMenuItem)sender).ToolTipText;
            if(!AcceptPlayList(s))AcceptFile(s);
        }

        private void listBox1_DrawItem(object sender, DrawItemEventArgs e)
        {
            if (e.Index == -1)
            {
            }
            else
            {
                e.DrawBackground();
                Brush b;
                if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                    b = SystemBrushes.HighlightText;
                else b = new SolidBrush(Color.Lime);
                Rectangle r = new Rectangle(e.Bounds.X + 15, e.Bounds.Y, e.Bounds.Width - 15 - 60, e.Bounds.Height);
                e.Graphics.DrawString((string)listBox1.Items[e.Index], e.Font, b, r);
                long l = (long)timeList[e.Index];
                int ii = (int)(l / 10000000);
                int h = ii / 3600;
                int m = (ii % 3600) / 60;
                int s = (ii % 3600) % 60;
                e.Graphics.DrawString(string.Format("{0:D2}:{1:D2}:{2:D2}", h, m, s), e.Font, b, e.Bounds.X + e.Bounds.Width - 60 + 5, e.Bounds.Y);
                if ((string)playList[e.Index] == currentFile)
                {
                    e.Graphics.FillEllipse(b, e.Bounds.X + 1, e.Bounds.Y + 1, 10, 10);
                }
                e.DrawFocusRectangle();
            }
        }

        private void menuNext_Click(object sender, EventArgs e)
        {
            int sel = playList.IndexOf(currentFile);
            if (sel < 0) return;
            if (sel < playList.Count - 1)
            {
                AcceptFile((string)playList[sel + 1]);
            }
            else
            {
                if ((string)playList[0] != currentFile) AcceptFile((string)playList[0]);
            }
        }

        private void menuPrev_Click(object sender, EventArgs e)
        {
            int sel = playList.IndexOf(currentFile);
            if (sel < 0) return;
            if (sel > 0)
            {
                AcceptFile((string)playList[sel - 1]);
            }
            else
            {
                if ((string)playList[playList.Count - 1] != currentFile) AcceptFile((string)playList[playList.Count - 1]);
            }
        }

        private void menuItemDelete_Click(object sender, EventArgs e)
        {
            int sel = listBox1.SelectedIndex;
            if (sel < 0) return;
            if ((string)playList[sel] == currentFile) return;
            playList.RemoveAt(sel);
            timeList.RemoveAt(sel);
            listBox1.Items.RemoveAt(sel);
        }

        private void menuItemUp_Click(object sender, EventArgs e)
        {
            int sel = listBox1.SelectedIndex;
            if (sel < 0) return;
            if (sel > 0)
            {
                object o = playList[sel];
                playList.RemoveAt(sel);
                playList.Insert(sel - 1, o);
                o = timeList[sel];
                timeList.RemoveAt(sel);
                timeList.Insert(sel - 1, o);
                o = listBox1.Items[sel];
                listBox1.Items.RemoveAt(sel);
                listBox1.Items.Insert(sel - 1, o);
                listBox1.SelectedIndex = sel - 1;
            }
        }

        private void menuItemDown_Click(object sender, EventArgs e)
        {
            int sel = listBox1.SelectedIndex;
            if (sel < 0) return;
            if (sel < playList.Count - 1)
            {
                object o = playList[sel + 1];
                playList.RemoveAt(sel + 1);
                playList.Insert(sel, o);
                o = timeList[sel + 1];
                timeList.RemoveAt(sel + 1);
                timeList.Insert(sel, o);
                o = listBox1.Items[sel + 1];
                listBox1.Items.RemoveAt(sel + 1);
                listBox1.Items.Insert(sel, o);
                listBox1.SelectedIndex = sel + 1;
            }
        }

        private void panelList_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void panelList_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                ArrayList list = new ArrayList();
                foreach (string fileName in (string[])e.Data.GetData(DataFormats.FileDrop))
                {
                   if(!AcceptPlayList(fileName))list.Add(fileName);
                }
                /*foreach (string fileName in (string[])(list.ToArray()))
                {
                    AcceptFile(fileName);
                }*///うまくいかない
                for (int i = 0; i < list.Count; i++) AcceptFile((string)list[i]);
            }
        }

        private void menuItemPlay_Click(object sender, EventArgs e)
        {
            AcceptFile((string)playList[listBox1.SelectedIndex]);
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex < 0)
            {
                buttonDelete.Enabled = false;
                menuItemDelete.Enabled = false;
                buttonPlayListUp.Enabled = false;
                menuItemUp.Enabled = false;
                buttonPlayListDown.Enabled = false;
                menuItemDown.Enabled = false;
            }
            else
            {
                buttonDelete.Enabled = (currentFile != (string)playList[listBox1.SelectedIndex]);
                menuItemDelete.Enabled = buttonDelete.Enabled;
                buttonPlayListDown.Enabled = (listBox1.SelectedIndex < playList.Count - 1);
                menuItemDown.Enabled = buttonPlayListDown.Enabled;
                buttonPlayListUp.Enabled = (listBox1.SelectedIndex > 0);
                menuItemUp.Enabled = buttonPlayListUp.Enabled;
            }
        }

        private void testToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (fl.BuildDvdGraph(null))
            {
                fl.RunGraph();
            }
        }
    }
}
